 /*
  * Copyright John E. Lloyd, 2000. All rights reserved. Permission
  * to use, copy, and modify, without fee, is granted for non-commercial 
  * and research purposes, provided that this copyright notice appears 
  * in all copies.
  *
  * This  software is distributed "as is", without any warranty, including 
  * any implied warranty of merchantability or fitness for a particular
  * use. The authors assume no responsibility for, and shall not be liable
  * for, any special, indirect, or consequential damages, or any damages
  * whatsoever, arising out of or in connection with the use of this
  * software.
  */

package cformat;

import java.io.*;
import javax.vecmath.GVector;

/**
  * Writer class to output primitive types using C <code>printf</code> style
  * formatting. For each primitive type (float, double, char, int, long,
  * String), there is a <code>printf</code> method which takes (as a first
  * argument) a <code>printf</code> style format string.
  * Because Java does not permit variable numbers of arguments, each
  * <code>printf</code> function accepts only one primitive type, and the
  * formats can correspondingly contain only one conversion
  * sequence.
  *
  * <p>
  * For example, suppose we wish to print a list of names and numbers
  * according to the printf format <code>"%s, %8.2f"</code>. This can
  * be done as follows:
  * <pre>
  *    String[] names = ...
  *    double[] score = ...
  *    PrintfWriter w = new PrintfWriter (writer);
  *    for (int i=0; i<1000; i++)
  *     { w.printf ("%s, ", names[i]);
  *       w.printf ("%8.2f", score[i]);
  *     }
  * </pre>
  * <p>
  * Instead of a format string, one can instead use a pre-parsed
  * {@link cformat.PrintfFormat PrintfFormat} object which
  * is created from a format string. This will be more efficient
  * when a lot of output is required:
  * <pre>
  *    double[] bigVector = new double[1000];
  *    PrintfWriter w = new PrintfWriter (writer);
  *    PrintfFormat fmt = new PrintfFormat ("%8.2f");
  *    for (int i=0; i<1000; i++)
  *     { w.printf (fmt, bigVector[i]);
  *     }
  * </pre>
  * <p>
  * Finally, {@link cformat.PrintfFormat PrintfFormat} can be used
  * to perform the conversion directly, via its
  * {@link cformat.PrintfFormat#tostr tostr} method:
  * <pre>
  *    PrintfFormat fmt = new PrintfFormat ("%8.2f");
  *    for (int i=0; i<1000; i++)
  *     { System.out.println (fmt.tostr(bigVector[i]));
  *     }
  * </pre>
  * @see PrintfFormat
  * @see PrintfStream
  * @author John E. Lloyd, Fall 2000
  */
public class PrintfWriter extends PrintWriter
{
	/** 
	  * Creates a PrintfWriter, without automatic line flushing,
	  * from an existing OutputStream.
	  * 
	  * @param out An output stream
	  */
	public PrintfWriter (OutputStream out)
	 { super (out);
	 }

	/**
	  * Creates a PrintfWriter from an existing OutputStream.
	  * 
	  * @param out An output stream
	  * @param autoFlush If true, specifies that output flushing will
	  * automatically occur when the println() methods are called.
	  */
	public PrintfWriter (OutputStream out, boolean autoFlush)
	 { super (out, autoFlush);
	 }

	/** 
	  * Creates a PrintfWriter, without automatic line flushing,
	  * from an existing Writer.
	  * 
	  * @param out A writer
	  */
	public PrintfWriter (Writer out)
	 { super (out);
	 }

	/** 
	  * Creates a PrintfWriter from an existing Writer.
	  * 
	  * @param out A writer
	  * @param autoFlush If true, specifies that output flushing will
	  * automatically occur when the println() methods are called.
	  */
	public PrintfWriter (Writer out, boolean autoFlush)
	 { super (out, autoFlush);
	 }

	/** 
	  * Prints a double in accordance with the supplied format string.
	  * 
	  * @param fs Format string
	  * @param x Double to output
	  * @throws IllegalArgumentException Malformed format string
	  */
	public void printf (String fs, double x)
	 { print (new PrintfFormat(fs).tostr(x));
	 }

	/** 
	  * Prints a float in accordance with the supplied format string.
	  * 
	  * @param fs Format string
	  * @param x Float to output
 	  * @throws IllegalArgumentException Malformed format string
	  */
	public void printf (String fs, float x)
	 { print (new PrintfFormat(fs).tostr(x));
	 }

	/** 
	  * Prints a long in accordance with the supplied format string.
	  * 
	  * @param fs Format string
	  * @param x Long to output
	  * @throws IllegalArgumentException Malformed format string
	  */
	public void printf (String fs, long x)
	 { print (new PrintfFormat(fs).tostr(x));
	 }

	/** 
	  * Prints an int in accordance with the supplied format string.
	  * 
	  * @param fs Format string
	  * @param x Int to output
	  * @throws IllegalArgumentException Malformed format string
	  */
	public void printf (String fs, int x)
	 { print (new PrintfFormat(fs).tostr(x));
	 }

	/** 
	  * Prints a String in accordance with the supplied format string.
	  * 
	  * @param fs Format string
	  * @param x String to output
	  * @throws IllegalArgumentException Malformed format string
	  */
	public void printf (String fs, String x)
	 { print (new PrintfFormat(fs).tostr(x));
	 }

	/** 
	  * Prints a char in accordance with the supplied format string.
	  * 
	  * @param fs Format string
	  * @param x Char to output
	  * @throws IllegalArgumentException Malformed format string
	  */
	public void printf (String fs, char x)
	 { print (new PrintfFormat(fs).tostr(x));
	 }

	/** 
	  * Prints a double in accordance with the supplied 
	  * PrintfFormat object.
	  * 
	  * @param fmt Formatting object
	  * @param x Double to output
	  * @see PrintfFormat
	  */
	public void printf (PrintfFormat fmt, double x)
	 { print (fmt.tostr(x));
	 }

	/** 
	  * Prints a float in accordance with the supplied 
	  * PrintfFormat object.
	  * 
	  * @param fmt Formatting object
	  * @param x Float to output
	  * @see PrintfFormat
	  */
	public void printf (PrintfFormat fmt, float x)
	 { print (fmt.tostr(x));
	 }

	/** 
	  * Prints a long in accordance with the supplied 
	  * PrintfFormat object.
	  * 
	  * @param fmt Formatting object
	  * @param x Long to output
	  * @see PrintfFormat
	  */
	public void printf (PrintfFormat fmt, long x)
	 { print (fmt.tostr(x));
	 }

	/** 
	  * Prints an int in accordance with the supplied 
	  * PrintfFormat object.
	  * 
	  * @param fmt Formatting object
	  * @param x Int to output
	  * @see PrintfFormat
	  */
	public void printf (PrintfFormat fmt, int x)
	 { print (fmt.tostr(x));
	 }

	/** 
	  * Prints a String in accordance with the supplied 
	  * PrintfFormat object.
	  * 
	  * @param fmt Formatting object
	  * @param x String to output
	  * @see PrintfFormat
	  */
	public void printf (PrintfFormat fmt, String x)
	 { print (fmt.tostr(x));
	 }

	/** 
	  * Prints a char in accordance with the supplied 
	  * PrintfFormat object.
	  * 
	  * @param fmt Formatting object
	  * @param x Char to output
	  * @see PrintfFormat
	  */
	public void printf (PrintfFormat fmt, char x)
	 { print (fmt.tostr(x));
	 }
}
